@@ -1,7 +1,7 @@
 #!/usr/bin/python3
 # -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
 #   MenuLibre - Advanced fd.o Compliant Menu Editor
-#   Copyright (C) 2012-2019 Sean Davis <smd.seandavis@gmail.com>
+#   Copyright (C) 2012-2021 Sean Davis <sean@bluesabre.org>
 #   Copyright (C) 2016-2018 OmegaPhil <OmegaPhil@startmail.com>
 #
 #   Portions of this file are adapted from Alacarte Menu Editor,
@@ -44,9 +44,34 @@ icon_theme = Gtk.IconTheme.get_default()
 menu_name = ""
 
 
+COL_NAME = 0
+COL_COMMENT = 1
+COL_EXEC = 2
+COL_CATEGORIES = 3
+COL_TYPE = 4
+COL_G_ICON = 5
+COL_ICON_NAME = 6
+COL_FILENAME = 7
+COL_EXPAND = 8
+COL_SHOW = 9
+
+
 def get_default_menu():
     """Return the filename of the default application menu."""
-    return '%s%s' % (util.getDefaultMenuPrefix(), 'applications.menu')
+    prefixes = [
+        util.getDefaultMenuPrefix(),
+        ''
+    ]
+    user_basedir = util.getUserMenuPath()
+    for prefix in prefixes:
+        filename = '%s%s' % (prefix, 'applications.menu')
+        user_dir = os.path.join(user_basedir, filename)
+        if os.path.exists(user_dir):
+            return filename
+        system_dir = util.getSystemMenuPath(filename)
+        if system_dir:
+            return filename
+    return None
 
 
 def load_fallback_icon(icon_size):
@@ -86,6 +111,7 @@ def menu_to_treestore(treestore, parent, menu_items):
     for item in menu_items:
         item_type = item[0]
         if item_type == MenuItemTypes.SEPARATOR:
+            executable = ""
             displayed_name = "<s>                    </s>"
             # Translators: Separator menu item
             tooltip = _("Separator")
@@ -95,6 +121,7 @@ def menu_to_treestore(treestore, parent, menu_items):
             icon_name = ""
             show = True
         else:
+            executable = item[2]['executable']
             displayed_name = escape(item[2]['display_name'])
             show = item[2]['show']
             tooltip = escapeText(item[2]['comment'])
@@ -104,8 +131,8 @@ def menu_to_treestore(treestore, parent, menu_items):
             icon_name = item[2]['icon_name']
 
         treeiter = treestore.append(
-            parent, [displayed_name, tooltip, categories, item_type,
-                     icon, icon_name, filename, False, show])
+            parent, [displayed_name, tooltip, executable, categories,
+                     item_type, icon, icon_name, filename, False, show])
 
         if item_type == MenuItemTypes.DIRECTORY:
             treestore = menu_to_treestore(treestore, treeiter, item[3])
@@ -115,12 +142,22 @@ def menu_to_treestore(treestore, parent, menu_items):
 
 def get_treestore():
     """Get the TreeStore implementation of the current menu."""
-    # Name, Comment, Categories, MenuItemType, GIcon (TreeView), icon-name,
-    # Filename, exp, show
-    treestore = Gtk.TreeStore(str, str, str, int, Gio.Icon, str, str, bool,
-                              bool)
-    menu = get_menus()[0]
-    return menu_to_treestore(treestore, None, menu)
+    treestore = Gtk.TreeStore(
+        str, # Name
+        str, # Comment
+        str, # Executable
+        str, # Categories
+        int, # MenuItemType
+        Gio.Icon, # GIcon
+        str, # icon-name
+        str, # Filename
+        bool, # Expand
+        bool  # Show
+    )
+    menus = get_menus()
+    if not menus:
+        return None
+    return menu_to_treestore(treestore, None, menus[0])
 
 
 def get_submenus(menu, tree_dir):
@@ -189,6 +226,8 @@ def get_submenus(menu, tree_dir):
 def get_menus():
     """Get the menus from the MenuEditor"""
     menu = MenuEditor()
+    if not menu.loaded:
+        return None
     structure = []
     toplevels = []
     global menu_name
@@ -230,12 +269,16 @@ def getUserMenuXml(tree):
 class MenuEditor(object):
     """MenuEditor class, adapted and minimized from Alacarte Menu Editor."""
 
+    loaded = False
+
     def __init__(self, basename=None):
         """init"""
 
         # Remember to keep menulibre-menu-validate's GMenu object creation
         # in-sync with this code
         basename = basename or get_default_menu()
+        if basename is None:
+            return
 
         self.tree = GMenu.Tree.new(basename,
                                    GMenu.TreeFlags.SHOW_EMPTY |
@@ -250,6 +293,8 @@ class MenuEditor(object):
         logger.debug("Using menu: %s" % self.path)
         self.loadDOM()
 
+        self.loaded = True
+
     def loadDOM(self):
         """loadDOM"""
         try:
