@@ -1,7 +1,7 @@
 #!/usr/bin/python3
 # -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
 #   MenuLibre - Advanced fd.o Compliant Menu Editor
-#   Copyright (C) 2012-2019 Sean Davis <smd.seandavis@gmail.com>
+#   Copyright (C) 2012-2021 Sean Davis <sean@bluesabre.org>
 #   Copyright (C) 2016-2018 OmegaPhil <OmegaPhil@startmail.com>
 #
 #   This program is free software: you can redistribute it and/or modify it
@@ -26,11 +26,14 @@ import tempfile
 
 from locale import gettext as _
 
+from gi import require_version
+require_version('Gtk', '3.0')
 from gi.repository import Gio, GLib, GObject, Gtk, Gdk, GdkPixbuf
 
 from . import MenulibreStackSwitcher, MenulibreIconSelection
 from . import MenulibreTreeview, MenulibreHistory, Dialogs
 from . import MenulibreXdg, util, MenulibreLog
+from . import MenuEditor
 from .util import MenuItemTypes, check_keypress, getBasename, getRelatedKeys
 from .util import escapeText, getCurrentDesktop, find_program
 import menulibre_lib
@@ -166,9 +169,17 @@ category_groups = {
     )
 }
 
-# Add support for X-Xfce-Toplevel items for XFCE environments.
+# DE-specific categories
 if util.getDefaultMenuPrefix() == 'xfce-':
-    category_groups['Xfce'] = ('X-Xfce-Toplevel',)
+    category_groups['Xfce'] = (
+        'X-XFCE', 'X-Xfce-Toplevel', 'X-XFCE-PersonalSettings', 'X-XFCE-HardwareSettings',
+        'X-XFCE-SettingsDialog', 'X-XFCE-SystemSettings'
+    )
+elif util.getDefaultMenuPrefix() == 'gnome-':
+    category_groups['GNOME'] = (
+        'X-GNOME-NetworkSettings', 'X-GNOME-PersonalSettings', 'X-GNOME-Settings-Panel',
+        'X-GNOME-Utilities'
+    )
 
 # Create a reverse-lookup
 category_lookup = dict()
@@ -246,7 +257,7 @@ class MenulibreWindow(Gtk.ApplicationWindow):
         # Set up the application editor
         self.configure_application_editor(builder)
 
-        # Set up the applicaton browser
+        # Set up the application browser
         self.configure_application_treeview(builder)
 
         # Determining paths of bad desktop files GMenu can't load - if some are
@@ -262,7 +273,7 @@ class MenulibreWindow(Gtk.ApplicationWindow):
             # dismissed.
             primary = _("MenuLibre cannot be run as root.")
 
-            docs_url = "https://wiki.bluesabre.org/menulibre_faq"
+            docs_url = "https://github.com/bluesabre/menulibre/wiki/Frequently-Asked-Questions"
 
             # Translators: This link goes to the online documentation with more
             # information.
@@ -276,6 +287,39 @@ class MenulibreWindow(Gtk.ApplicationWindow):
             dialog.run()
             sys.exit(1)
 
+    def menu_load_failure(self):
+        primary = _("MenuLibre failed to load.")
+
+        docs_url = "https://github.com/bluesabre/menulibre/wiki/Frequently-Asked-Questions"
+
+        # Translators: This link goes to the online documentation with more
+        # information.
+        secondary = _("The default menu could not be found. Please see the "
+                        "<a href='%s'>online documentation</a> "
+                        "for more information.") % docs_url
+
+        secondary += "\n\n<big><b>%s</b></big>" % _("Diagnostics")
+
+        diagnostics = util.getMenuDiagnostics()
+        for k, v in diagnostics.items():
+            secondary += "\n<b>%s</b>: %s" % (k, v)
+
+        dialog = Gtk.MessageDialog(None, 0, Gtk.MessageType.ERROR,
+                                    Gtk.ButtonsType.CLOSE, primary)
+        dialog.format_secondary_markup(secondary)
+
+        try:
+            box = dialog.get_children()[0]
+            box = box.get_children()[0]
+            box = box.get_children()[1]
+            label = box.get_children()[1]
+            label.set_selectable(True)
+        except AttributeError:
+            pass
+
+        dialog.run()
+        sys.exit(1)
+
     def configure_application_window(self, builder, app):
         """Glade is currently unable to create a GtkApplicationWindow.  This
         function takes the GtkWindow from the UI file and reparents the
@@ -587,6 +631,8 @@ class MenulibreWindow(Gtk.ApplicationWindow):
     def configure_application_treeview(self, builder):
         """Configure the menu-browsing GtkTreeView."""
         self.treeview = MenulibreTreeview.Treeview(self, builder)
+        if not self.treeview.loaded:
+            self.menu_load_failure()
         treeview = self.treeview.get_treeview()
         treeview.set_search_entry(self.search_box)
         self.search_box.connect('changed', self.on_app_search_changed,
@@ -671,12 +717,14 @@ class MenulibreWindow(Gtk.ApplicationWindow):
             'StartupWMClass': builder.get_object('entry_StartupWMClass'),
             'Implements': builder.get_object('entry_Implements'),
             'Hidden': builder.get_object('switch_Hidden'),
-            'DBusActivatable': builder.get_object('switch_DBusActivatable')
+            'DBusActivatable': builder.get_object('switch_DBusActivatable'),
+            'PrefersNonDefaultGPU': builder.get_object('switch_PrefersNonDefaultGPU'),
+            'X-GNOME-UsesNotifications': builder.get_object('switch_UsesNotifications')
         }
 
         # Configure the switches
         for widget_name in ['Terminal', 'StartupNotify', 'NoDisplay', 'Hidden',
-                            'DBusActivatable']:
+                            'DBusActivatable', 'PrefersNonDefaultGPU']:
             widget = self.widgets[widget_name]
             widget.connect('notify::active', self.on_switch_toggle,
                            widget_name)
@@ -712,6 +760,19 @@ class MenulibreWindow(Gtk.ApplicationWindow):
             button.connect('focus-out-event',
                            self.on_NameCommentIcon_focus_out_event)
 
+        for widget_name in ['Name', 'Comment']:
+            entry = builder.get_object('entry_%s' % widget_name)
+
+            # Commit changes to entries when focusing out.
+            entry.connect('focus-out-event',
+                          self.on_entry_focus_out_event,
+                          widget_name)
+
+            # Enable saving on any edit with an Entry.
+            entry.connect("changed",
+                          self.on_entry_changed,
+                          widget_name)
+
         for widget_name in ['Exec', 'Path', 'GenericName', 'TryExec',
                             'OnlyShowIn', 'NotShowIn', 'MimeType', 'Keywords',
                             'StartupWMClass', 'Implements']:
@@ -906,7 +967,7 @@ class MenulibreWindow(Gtk.ApplicationWindow):
                 rows.append(row_data)
 
         if sort:
-            rows = sorted(rows, key=lambda row_data: row_data[key_columns[0]])
+            rows = sorted(rows, key=lambda row_data: row_data[key_columns[MenuEditor.COL_NAME]])
 
         model.clear()
         for row in rows:
@@ -1195,7 +1256,8 @@ class MenulibreWindow(Gtk.ApplicationWindow):
                     'NoDisplay', 'GenericName', 'TryExec',
                     'OnlyShowIn', 'NotShowIn', 'MimeType',
                     'Keywords', 'StartupWMClass', 'Implements', 'Categories',
-                    'Hidden', 'DBusActivatable']:
+                    'Hidden', 'DBusActivatable', 'PrefersNonDefaultGPU',
+                    'X-GNOME-UsesNotifications']:
                     self.set_value(key, None)
 
         # Clear the Actions and Icon.
@@ -1203,7 +1265,7 @@ class MenulibreWindow(Gtk.ApplicationWindow):
         self.set_value('Icon', None, store=True)
 
         model, row_data = self.treeview.get_selected_row_data()
-        item_type = row_data[3]
+        item_type = row_data[MenuEditor.COL_TYPE]
 
         # If the selected row is a separator, hide the editor.
         if item_type == MenuItemTypes.SEPARATOR:
@@ -1225,15 +1287,15 @@ class MenulibreWindow(Gtk.ApplicationWindow):
                 basename = getBasename(filename)
                 filename = util.getSystemLauncherPath(basename)
                 if filename is not None:
-                    row_data[6] = filename
+                    row_data[MenuEditor.COL_FILENAME] = filename
                     self.treeview.update_launcher_instances(filename, row_data)
 
             if new_launcher or (filename is not None):
                 self.editor.show()
-                displayed_name = row_data[0]
-                comment = row_data[1]
+                displayed_name = row_data[MenuEditor.COL_NAME]
+                comment = row_data[MenuEditor.COL_COMMENT]
 
-                self.set_value('Icon', row_data[5], store=True)
+                self.set_value('Icon', row_data[MenuEditor.COL_ICON_NAME], store=True)
                 self.set_value('Name', displayed_name, store=True)
                 self.set_value('Comment', comment, store=True)
                 self.set_value('Filename', filename, store=True)
@@ -1642,8 +1704,9 @@ class MenulibreWindow(Gtk.ApplicationWindow):
         icon_name = "applications-other"
         icon = Gio.ThemedIcon.new(icon_name)
         filename = None
-        new_row_data = [name, comment, categories, item_type, icon, icon_name,
-                        filename, True]
+        executable = ""
+        new_row_data = [name, comment, executable, categories, item_type, icon,
+                        icon_name, filename, True]
 
         model, parent_data = self.treeview.get_parent_row_data()
         model, row_data = self.treeview.get_selected_row_data()
@@ -1654,7 +1717,7 @@ class MenulibreWindow(Gtk.ApplicationWindow):
 
         # Add to the treeview on the current level or as a child of a selected
         # directory
-        dir_selected = row_data[3] == MenuItemTypes.DIRECTORY
+        dir_selected = row_data[MenuEditor.COL_TYPE] == MenuItemTypes.DIRECTORY
         if dir_selected:
             self.treeview.add_child(new_row_data)
         else:
@@ -1664,14 +1727,14 @@ class MenulibreWindow(Gtk.ApplicationWindow):
         # directory, so the resulting item will be placed at the current level
         # fetch the parent's categories
         if parent_data is not None and not dir_selected:
-            categories = util.getRequiredCategories(parent_data[6])
+            categories = util.getRequiredCategories(parent_data[MenuEditor.COL_FILENAME])
 
         elif parent_data is not None and dir_selected:
 
             # A directory lower than the top-level has been selected - the
             # launcher will be added into it (e.g. as the first item),
             # therefore it essentially has a parent of the current selection
-            categories = util.getRequiredCategories(row_data[6])
+            categories = util.getRequiredCategories(row_data[MenuEditor.COL_FILENAME])
 
         else:
 
@@ -1695,8 +1758,9 @@ class MenulibreWindow(Gtk.ApplicationWindow):
         icon_name = "folder"
         icon = Gio.ThemedIcon.new(icon_name)
         filename = None
-        row_data = [name, comment, categories, item_type, icon, icon_name,
-                    filename, True, True]
+        executable = ""
+        row_data = [name, comment, executable, categories, item_type, icon,
+                    icon_name, filename, True, True]
 
         self.treeview.append(row_data)
 
@@ -1714,8 +1778,9 @@ class MenulibreWindow(Gtk.ApplicationWindow):
         icon_name = ""
         item_type = MenuItemTypes.SEPARATOR
         filename = None
-        row_data = [name, tooltip, categories, item_type, icon, icon_name,
-                    filename, False, True]
+        executable = ""
+        row_data = [name, tooltip, executable, categories, item_type, icon,
+                    icon_name, filename, False, True]
 
         self.treeview.append(row_data)
 
@@ -1792,7 +1857,7 @@ class MenulibreWindow(Gtk.ApplicationWindow):
 
         if not temp:
             model, row_data = self.treeview.get_selected_row_data()
-            item_type = row_data[3]
+            item_type = row_data[MenuEditor.COL_TYPE]
 
             model, parent_data = self.treeview.get_parent_row_data()
 
@@ -1804,7 +1869,7 @@ class MenulibreWindow(Gtk.ApplicationWindow):
             if parent_data is not None:
                 # Parent was found, take its categories.
                 required_categories = util.getRequiredCategories(
-                    parent_data[6])
+                    parent_data[MenuEditor.COL_FILENAME])
             else:
                 # Parent was not found, this is a toplevel category
                 required_categories = util.getRequiredCategories(None)
@@ -1841,11 +1906,12 @@ class MenulibreWindow(Gtk.ApplicationWindow):
         # Update the selected iter with the new details.
         name = self.get_value('Name')
         comment = self.get_value('Comment')
+        executable = self.get_value('Exec')
         categories = self.get_value('Categories')
         icon_name = self.get_value('Icon')
         hidden = self.get_value('Hidden') or self.get_value('NoDisplay')
-        self.treeview.update_selected(name, comment, categories, item_type,
-                                      icon_name, filename, not hidden)
+        self.treeview.update_selected(name, comment, executable, categories,
+                                      item_type, icon_name, filename, not hidden)
         self.history.clear()
 
         # Update all instances
@@ -1910,8 +1976,8 @@ class MenulibreWindow(Gtk.ApplicationWindow):
 
         # Update all instances
         model, row_data = self.treeview.get_selected_row_data()
-        row_data[2] = ';'.join(categories)
-        row_data[6] = save_filename
+        row_data[MenuEditor.COL_CATEGORIES] = ';'.join(categories)
+        row_data[MenuEditor.COL_FILENAME] = save_filename
         self.treeview.update_launcher_instances(original_filename, row_data)
 
     def update_launcher_category_dirs(self):  # noqa
@@ -1931,11 +1997,11 @@ class MenulibreWindow(Gtk.ApplicationWindow):
         # user has explicitly deleted) will be added, so this shouldn't be a
         # problem
         model, row_data = self.treeview.get_selected_row_data()
-        if row_data[2]:
-            categories = row_data[2].split(';')[:-1]
+        if row_data[MenuEditor.COL_CATEGORIES]:
+            categories = row_data[MenuEditor.COL_CATEGORIES].split(';')[:-1]
         else:
             categories = []
-        filename = row_data[6]
+        filename = row_data[MenuEditor.COL_FILENAME]
 
         required_category_directories = set()
 
@@ -1950,21 +2016,21 @@ class MenulibreWindow(Gtk.ApplicationWindow):
             # regardless...
             _, parent = self.treeview.get_parent(model, instance)
             if (parent is not None and
-                    model[parent][3] == MenuItemTypes.DIRECTORY):
+                    model[parent][MenuEditor.COL_TYPE] == MenuItemTypes.DIRECTORY):
 
                 # Any direct parents are required directories.
-                required_category_directories.add(model[parent][0])
+                required_category_directories.add(model[parent][MenuEditor.COL_NAME])
 
                 # Adding if the directory returned is top level
                 _, parent_parent = self.treeview.get_parent(model, parent)
                 if parent_parent is None:
-                    launchers_in_top_level_dirs[model[parent][0]] = instance
+                    launchers_in_top_level_dirs[model[parent][MenuEditor.COL_NAME]] = instance
 
         # Obtaining a lookup of top-level directories -> iters
         top_level_dirs = {}
         for row in model:
-            if row[3] == MenuItemTypes.DIRECTORY:
-                top_level_dirs[row[0]] = model.get_iter(row.path)
+            if row[MenuEditor.COL_TYPE] == MenuItemTypes.DIRECTORY:
+                top_level_dirs[row[MenuEditor.COL_NAME]] = model.get_iter(row.path)
 
         # Looping through all set categories - category specified is at maximum
         # detail level, this needs to be converted to the parent group name,
@@ -1998,7 +2064,7 @@ class MenulibreWindow(Gtk.ApplicationWindow):
 
             # Removing selected launcher from the UI if it is in the current
             # directory, otherwise just from the model
-            if parent_data is not None and directory_name == parent_data[0]:
+            if parent_data is not None and directory_name == parent_data[MenuEditor.COL_NAME]:
                 self.treeview.remove_selected(True)
 
             else:
@@ -2107,8 +2173,8 @@ class MenulibreWindow(Gtk.ApplicationWindow):
     def on_delete_cb(self, widget):
         """Delete callback function."""
         model, row_data = self.treeview.get_selected_row_data()
-        name = row_data[0]
-        item_type = row_data[3]
+        name = row_data[MenuEditor.COL_NAME]
+        item_type = row_data[MenuEditor.COL_TYPE]
 
         # Prepare the strings
         if item_type == MenuItemTypes.SEPARATOR:
