@@ -1,7 +1,7 @@
 #!/usr/bin/python3
 # -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
 #   MenuLibre - Advanced fd.o Compliant Menu Editor
-#   Copyright (C) 2012-2019 Sean Davis <smd.seandavis@gmail.com>
+#   Copyright (C) 2012-2021 Sean Davis <sean@bluesabre.org>
 #   Copyright (C) 2016 OmegaPhil <OmegaPhil@startmail.com>
 #
 #   This program is free software: you can redistribute it and/or modify it
@@ -203,7 +203,7 @@ def model_to_xml_menus(model, model_parent=None, menu_parent=None):
         treeiter = model.iter_nth_child(model_parent, n_child)
 
         # Extract the menu item details.
-        name, comment, categories, item_type, gicon, icon, desktop, expanded, \
+        name, comment, executable, categories, item_type, gicon, icon, desktop, expanded, \
             show = model[treeiter][:]
 
         if item_type == MenuItemTypes.DIRECTORY:
@@ -250,7 +250,7 @@ def model_to_xml_includes(model, model_parent=None, menu_parent=None):
         treeiter = model.iter_nth_child(model_parent, n_child)
 
         # Extract the menu item details.
-        name, comment, categories, item_type, gicon, icon, desktop, expanded, \
+        name, comment, executable, categories, item_type, gicon, icon, desktop, expanded, \
             show = model[treeiter][:]
 
         if desktop is None:
@@ -288,7 +288,7 @@ def model_to_xml_layout(model, model_parent=None, menu_parent=None,  # noqa
         treeiter = model.iter_nth_child(model_parent, n_child)
 
         # Extract the menu item details.
-        name, comment, categories, item_type, gicon, icon, desktop, expanded, \
+        name, comment, executable, categories, item_type, gicon, icon, desktop, expanded, \
             show = model[treeiter][:]
 
         if item_type == MenuItemTypes.DIRECTORY:
