@@ -1,7 +1,7 @@
 #!/usr/bin/python3
 # -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
 #   MenuLibre - Advanced fd.o Compliant Menu Editor
-#   Copyright (C) 2012-2019 Sean Davis <smd.seandavis@gmail.com>
+#   Copyright (C) 2012-2021 Sean Davis <sean@bluesabre.org>
 #   Copyright (C) 2017-2018 OmegaPhil <OmegaPhil@startmail.com>
 #
 #   This program is free software: you can redistribute it and/or modify it
@@ -59,6 +59,8 @@ MenuItemKeys = (
     ("OnlyShowIn", list, False, (0, 1, 2)),
     ("NotShowIn", list, False, (0, 1, 2)),
     ("DBusActivatable", bool, False, (0,)),
+    ("PrefersNonDefaultGPU", bool, False, (0,)),
+    ("X-GNOME-UsesNotifications", bool, False, (0,)),
     ("TryExec", str, False, (0,)),
     ("Exec", str, True, (0,)),
     ("Path", str, False, (0,)),
@@ -205,6 +207,34 @@ def getDefaultMenuPrefix(): # noqa
     return prefix
 
 
+def getMenuDiagnostics():
+    diagnostics = {}
+    keys = [
+        "XDG_CURRENT_DESKTOP",
+        "XDG_MENU_PREFIX",
+        "DESKTOP_SESSION",
+        "KDE_SESSION_VERSION"
+    ]
+    for k in keys:
+        diagnostics[k] = os.environ.get(k, "None")
+
+    menu_dirs = [
+        getUserMenuPath()
+    ]
+    for path in GLib.get_system_config_dirs():
+        menu_dirs.append(os.path.join(path, 'menus'))
+    menus = []
+    for menu_dir in menu_dirs:
+        for filename in os.listdir(menu_dir):
+            if filename.endswith(".menu"):
+                menus.append(os.path.join(menu_dir, filename))
+    menus.sort()
+
+    diagnostics["MENUS"] = ", ".join(menus)
+
+    return diagnostics
+
+
 def getItemPath(file_id):
     """Return the path to the system-installed .desktop file."""
     for path in GLib.get_system_data_dirs():
@@ -643,7 +673,15 @@ def validate_desktop_file(desktop_file):  # noqa
         # missing in a failing desktop file.
         return _('%s key not found') % 'Type'
 
-    if type_key != GLib.KEY_FILE_DESKTOP_TYPE_APPLICATION:
+    valid_type_keys = [
+        GLib.KEY_FILE_DESKTOP_TYPE_APPLICATION,
+        GLib.KEY_FILE_DESKTOP_TYPE_LINK,
+        GLib.KEY_FILE_DESKTOP_TYPE_DIRECTORY,
+        # KDE-specific types
+        # https://specifications.freedesktop.org/desktop-entry-spec/latest/apb.html
+        "ServiceType", "Service", "FSDevice"
+    ]
+    if type_key not in valid_type_keys:
         # Translators: This error is displayed when a failing desktop file
         # has an invalid value for the provided key.
         return (_('%s value is invalid - currently \'%s\', should be \'%s\'')
@@ -681,7 +719,10 @@ def validate_desktop_file(desktop_file):  # noqa
         return (_('%s program \'%s\' is not a valid shell command '
                   'according to GLib.shell_parse_argv, error: %s')
                 % ('Exec', exec_key, e))
+    
+    if type_key == "Service":
+        return False # KDE services are not displayed in the menu
 
     # Translators: This error is displayed for a failing desktop file where
     # errors were detected but the file seems otherwise valid.
-    return _('Unknown error. Desktop file appears to be are valid.')
+    return _('Unknown error. Desktop file appears to be valid.')
